# Design

## ドキュメント

### ソーステキストファースト

ドキュメントの可読性の基準はソーステキスト(Markdownテキスト)でありレンダリング後の可読性はオプションである。
よって構文はレンダリングを要さず直接意味付けの役割を果たさなければならない。

ゆえにすべての構文は直感に従い意味を解釈できる。

### ドキュメント指向

ドキュメントはHTMLドキュメントに要求される人的操作性を損なってはならない。
よって構文は次の操作的機能を備えなければならない。

- 参照性
  - 主要項目を識別子により参照可能である
- 編集性
  - 主要項目の識別子を任意の識別子で固定可能である
  - 識別子は記述位置に依存せず可搬である

ゆえに構文は識別子の付与および参照構文を備える。

### ユニバーサルポータビリティ

ドキュメントは異なるサイトやツール上でも正しく表示されなければならない。
よってAPIはパラメータによりパーサーの動作に影響を与えてはならない。

ゆえにAPIはソーステキストのみを唯一のパラメータとして受け取る。

### サードパーティドキュメントの共有および表示可能性

ドキュメントはそのセキュリティ上の信頼性にかかわらず共有および表示できなければならない。
よってパーサーおよびレンダラーはXSSをはじめとする悪意あるコードの注入に対して安全でなければならない。

ゆえにパーサーは設計レベルでXSS安全である。
ただし外部メディアの表示等安全性を外部サービスに依存する部分はこの限りではない。

## 構文

### 引用文のモジュラリティ

引用構文はコピー＆ペーストにより挿入されたテキストを無編集で使用できなければならない。

ゆえに引用構文は引用文をパースしない。

### 自動附番

ドキュメント内の参照はドキュメントの編集を妨げないよう参照先および参照元ともに順序に非依存でなければならない。

ゆえに参照箇所に実体を記述する注釈構文および識別に文字列を使用する図表構文を採用し、その表示方法は任意とする。
脚注構文は附番が手作業となり参照と実体の対応の管理が困難であるため採用せず注釈構文により生成可能とする。

### 羅列的知識への非依存

構文はその使用のために羅列的知識を求めてはならない。

ゆえに構文はHTMLエンティティの参照の有効性を検査しない。

### 閲覧者のマークアップ方法の確保

構文とその表現は閲覧者の基本的なマークアップを妨げてはならない。

ゆえに構文は下線、打ち消し線、ハイライトのための構文を備えない。

### テンプレート対応

ソーステキストは自身をテンプレートとして別のソーステキストを生成するための構文を使用できなければならない。
またテンプレートとなるソーステキストもドキュメントとして正しく表示できなければならない。
よって構文はテンプレートのソーステキストとドキュメント間での表示上の一貫性のための構文を備えなければならない。

ゆえに構文はテンプレート構文を備える。

### 編集中のソーステキストのレンダリング結果の表示安定性

レンダリング結果の表示はソーステキストの編集にともなう高頻度または広範囲にわたる変化により視覚的負担をかけてはならない。
よって構文は編集過程で生じる不完全な構文を最大限補完してパースしなければならない。

ゆえにすべてのブロックレベルの構文は順次的編集過程で生じるすべての不完全な構文を補完してパースする。
またインラインのレンダリングに含まれる高価な処理はキャッシュにより即時化される。

### 構文冗長性

構文の消費する文字列ならびに正常なパース結果の表示上および機能上の意味は構文拡張の前後において不変でなければならない。
よって構文の種類や属性等のパラメータは値でなく書式で制約しなければならない。
また不正ないし未知のパラメータを含む入力文字列は誤入力および出力の可変性を通知するパース結果を出力しなければならない。

### 構文拡張

構文拡張は次のいずれかの要求に対応するものでなければならない。

- HTML上の表示形式への対応
- HTML上で固有の視覚表現を持つ意味付けへの対応
- HTMLドキュメントに要求される操作的機能の充足
- 構文個別の要件の充足

よって外観や制御のために構文を追加してはならない。
また他の構文と同じ視覚表現を持ち意味付けのみ異なるHTMLタグを追加してはならない。

## スケーラビリティ

パーサーおよびレンダラーはドキュメントのサイズにより以下の利用性を損なってはならない。

### 閲覧性

ドキュメントは即時ないし順次閲覧できなければならない。
よってドキュメントは全体のパースを待たず一定時間内に表示を開始しなければならない。
また高価な処理は必要時まで遅延しなければならない。

ゆえにパーサーはパースおよびその結果の返却を逐次的に行う。
またパーサーは外部リソースのリクエストその他の高価な処理を自身で行わずレンダラーに委ねる。

### 参照性

ドキュメントは各要素への参照、参照先への移動、および参照元への復帰方法を提供しなければならない。

ゆえにドキュメントはすべてのセクション、用語、図表および注釈をリンクから参照できる。

### 編集性

ドキュメントはそのサイズにかかわらず編集が容易でなければならない。
よってパースおよびレンダリングは差分更新により行わなければならない。

ゆえにパーサーは差分更新を行い、構文はこのために空行によるブロックの分割を必須とする。
